@@ -38,14 +38,34 @@ The schema for each option is defined using the
 Option Types
 ------------
 
-Options can have arbitrary types via the ``type`` constructor to :class:`Opt`.
-The type constructor is a callable object that takes a string and either
-returns a value of that particular type or raises ValueError if the value can
-not be converted.
+Options can have arbitrary types via the `type` parameter to the :class:`Opt`
+constructor. The `type` parameter is a callable object that takes a string and
+either returns a value of that particular type or raises :class:`ValueError` if
+the value can not be converted.
 
-There are predefined types in :class:`oslo_config.cfg` : strings,
-integers, floats, booleans, IP address, port, sub-command, lists,
-'multi strings' and 'key/value pairs' (dictionary) ::
+For convenience, there are predefined option subclasses in
+:mod:`oslo_config.cfg` that set the option `type` as in the following table:
+
+====================================  ======
+Type                                  Option
+====================================  ======
+:class:`oslo_config.types.String`     - :class:`oslo_config.cfg.StrOpt`
+                                      - :class:`oslo_config.cfg.SubCommandOpt`
+:class:`oslo_config.types.Boolean`    :class:`oslo_config.cfg.BoolOpt`
+:class:`oslo_config.types.Integer`    - :class:`oslo_config.cfg.IntOpt`
+                                      - :class:`oslo_config.cfg.PortOpt`
+:class:`oslo_config.types.Float`      :class:`oslo_config.cfg.FloatOpt`
+:class:`oslo_config.types.List`       :class:`oslo_config.cfg.ListOpt`
+:class:`oslo_config.types.Dict`       :class:`oslo_config.cfg.DictOpt`
+:class:`oslo_config.types.IPAddress`  :class:`oslo_config.cfg.IPOpt`
+====================================  ======
+
+For :class:`oslo_config.cfg.MultiOpt` the `item_type` parameter defines
+the type of the values. For convenience, :class:`oslo_config.cfg.MultiStrOpt`
+is :class:`~oslo_config.cfg.MultiOpt` with the `item_type` parameter set to
+:class:`oslo_config.types.MultiString`.
+
+The following example defines options using the convenience classes::
 
     enabled_apis_opt = cfg.ListOpt('enabled_apis',
                                    default=['ec2', 'osapi_compute'],
@@ -622,7 +642,7 @@ class Opt(object):
     :param name: the option's name
     :param type: the option's type. Must be a callable object that takes string
                  and returns converted and validated value
-    :param dest: the name of the corresponding ConfigOpts property
+    :param dest: the name of the corresponding :class:`.ConfigOpts` property
     :param short: a single character CLI option name
     :param default: the default value of the option
     :param positional: True if the option is a positional CLI argument
@@ -632,10 +652,13 @@ class Opt(object):
     :param required: true if a value must be supplied for this option
     :param deprecated_name: deprecated name option.  Acts like an alias
     :param deprecated_group: the group containing a deprecated alias
-    :param deprecated_opts: array of DeprecatedOpt(s)
+    :param deprecated_opts: list of :class:`.DeprecatedOpt`
     :param sample_default: a default string for sample config files
     :param deprecated_for_removal: indicates whether this opt is planned for
                                    removal in a future release
+    :param deprecated_reason: indicates why this opt is planned for removal in
+                              a future release. Silently ignored if
+                              deprecated_for_removal is False
 
     An Opt object has no public methods, but has a number of public properties:
 
@@ -651,7 +674,8 @@ class Opt(object):
 
     .. py:attribute:: dest
 
-        the (hyphen-less) ConfigOpts property which contains the option value
+        the (hyphen-less) :class:`.ConfigOpts` property which contains the
+        option value
 
     .. py:attribute:: short
 
@@ -687,8 +711,10 @@ class Opt(object):
        Added *deprecated_for_removal* parameter.
 
     .. versionchanged:: 2.7
-
        An exception is now raised if the default value has the wrong type.
+
+    .. versionchanged:: 3.2
+       Added *deprecated_reason* parameter.
     """
     multi = False
 
@@ -697,7 +723,7 @@ class Opt(object):
                  secret=False, required=False,
                  deprecated_name=None, deprecated_group=None,
                  deprecated_opts=None, sample_default=None,
-                 deprecated_for_removal=False):
+                 deprecated_for_removal=False, deprecated_reason=None):
         if name.startswith('_'):
             raise ValueError('illegal name %s with prefix _' % (name,))
         self.name = name
@@ -722,6 +748,7 @@ class Opt(object):
         self.secret = secret
         self.required = required
         self.deprecated_for_removal = deprecated_for_removal
+        self.deprecated_reason = deprecated_reason
         self._logged_deprecation = False
         if deprecated_name is not None:
             deprecated_name = deprecated_name.replace('-', '_')
@@ -818,6 +845,7 @@ class Opt(object):
         :param kwargs: the keyword arguments for add_argument()
         :param prefix: an optional prefix to prepend to the opt name
         :param positional: whether the option is a positional CLI argument
+        :param deprecated_names: list of deprecated option names
         """
         def hyphen(arg):
             return arg if not positional else ''
@@ -849,7 +877,7 @@ class Opt(object):
         options added to argparse.
 
         :param group: an optional group
-        :param kwargs: optional keyword arguments to add to
+        :param \*\*kwargs: optional keyword arguments to add to
         :returns: a dict of keyword arguments
         """
         if not self.positional:
@@ -997,7 +1025,7 @@ class StrOpt(Opt):
                         between 'choices' or 'regex' will be ignored.
     :param max_length: If positive integer, the value must be less than or
                        equal to this parameter.
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     .. versionchanged:: 2.7
        Added *quotes* parameter
@@ -1034,7 +1062,7 @@ class BoolOpt(Opt):
     In config files, boolean values are cast with Boolean type.
 
     :param name: the option's name
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
     """
 
     def __init__(self, name, **kwargs):
@@ -1090,7 +1118,7 @@ class IntOpt(Opt):
     :param name: the option's name
     :param min: minimum value the integer can take
     :param max: maximum value the integer can take
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     .. versionchanged:: 1.15
 
@@ -1109,7 +1137,7 @@ class FloatOpt(Opt):
     Option with ``type`` :class:`oslo_config.types.Float`
 
     :param name: the option's name
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
     """
 
     def __init__(self, name, **kwargs):
@@ -1125,7 +1153,7 @@ class ListOpt(Opt):
     :param name: the option's name
     :param item_type: type of items (see :class:`oslo_config.types`)
     :param bounds: if True the value should be inside "[" and "]" pair
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     .. versionchanged:: 2.5
        Added *item_type* and *bounds* parameters.
@@ -1145,7 +1173,7 @@ class DictOpt(Opt):
     Option with ``type`` :class:`oslo_config.types.Dict`
 
     :param name: the option's name
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     .. versionadded:: 1.2
     """
@@ -1163,7 +1191,7 @@ class IPOpt(Opt):
     :param name: the option's name
     :param version: one of either ``4``, ``6``, or ``None`` to specify
        either version.
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     .. versionadded:: 1.4
     """
@@ -1180,15 +1208,35 @@ class PortOpt(Opt):
     Option with ``type`` :class:`oslo_config.types.Integer`
 
     :param name: the option's name
-    :param min: minimum port number the option can take
-    :param max: maximum port number the option can take
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param choices: Optional sequence of valid values.
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     .. versionadded:: 2.6
+    .. versionchanged:: 3.2
+       Added *choices* parameter.
     """
+    PORT_MIN = 1
+    PORT_MAX = 65535
 
-    def __init__(self, name, **kwargs):
-        type = types.Integer(min=1, max=65535, type_name='port value')
+    def __init__(self, name, choices=None, **kwargs):
+
+        # choices and min/max are mutally exclusive for Integer type. So check
+        # if choice are in the range of min/max and only assign choices to
+        # Integer type.
+        if choices is not None:
+            invalid_choices = []
+            for choice in choices:
+                if not self.PORT_MIN <= choice <= self.PORT_MAX:
+                    invalid_choices.append(six.text_type(choice))
+            if invalid_choices:
+                raise ValueError("'Choices' values %(choices)s should be in "
+                                 "the range of %(min)d and %(max)d" %
+                                 {'choices': ', '.join(invalid_choices),
+                                  'min': self.PORT_MIN, 'max': self.PORT_MAX})
+            type = types.Integer(choices=choices, type_name='port value')
+        else:
+            type = types.Integer(min=self.PORT_MIN, max=self.PORT_MAX,
+                                 type_name='port value')
         super(PortOpt, self).__init__(name, type=type, **kwargs)
 
 
@@ -1201,7 +1249,7 @@ class MultiOpt(Opt):
 
     :param name: the option's name
     :param item_type: Type of items (see :class:`oslo_config.types`)
-    :param **kwargs: arbitrary keyword arguments passed to :class:`Opt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`Opt`
 
     For example::
 
@@ -1238,7 +1286,7 @@ class MultiStrOpt(MultiOpt):
     type.
 
     :param name: the option's name
-    :param **kwargs: arbitrary keyword arguments passed to :class:`MultiOpt`
+    :param \*\*kwargs: arbitrary keyword arguments passed to :class:`MultiOpt`
     """
 
     def __init__(self, name, **kwargs):
@@ -1263,7 +1311,7 @@ class SubCommandOpt(Opt):
     sub-parser arguments available as additional attributes.
 
     :param name: the option's name
-    :param dest: the name of the corresponding ConfigOpts property
+    :param dest: the name of the corresponding :class:`.ConfigOpts` property
     :param handler: callable which is supplied subparsers object when invoked
     :param title: title of the sub-commands group in help output
     :param description: description of the group in help output
@@ -1563,6 +1611,12 @@ class ConfigParser(iniparser.BaseParser):
 
 
 class MultiConfigParser(object):
+    """A ConfigParser which handles multi-opts.
+
+    All methods in this class which accept config names should treat a section
+    name of None as 'DEFAULT'.
+    """
+
     _deprecated_opt_message = ('Option "%s" from group "%s" is deprecated. '
                                'Use option "%s" from group "%s".')
 
@@ -1608,10 +1662,13 @@ class MultiConfigParser(object):
         :param names: a list of (section, name) tuples
         :param multi: a boolean indicating whether to return multiple values
         :param normalized: whether to normalize group names to lowercase
+        :param current_name: current name in tuple being checked
         """
         rvalue = []
 
         def normalize(name):
+            if name is None:
+                name = 'DEFAULT'
             return _normalize_group_name(name) if normalized else name
 
         names = [(normalize(section), name) for section, name in names]
@@ -1645,12 +1702,7 @@ class MultiConfigParser(object):
                     If the name param matches any entries in this list a
                     deprecation warning will be logged.
         """
-        # Opts in the DEFAULT group may come in with a group name of either
-        # 'DEFAULT' or None.  Force them all to 'DEFAULT' since that's a more
-        # user-friendly form.
-        deprecated_names = set((g or 'DEFAULT', n) for (g, n) in deprecated)
-        name = (name[0] or 'DEFAULT', name[1])
-        if name in deprecated_names and name not in self._emitted_deprecations:
+        if name in deprecated and name not in self._emitted_deprecations:
             self._emitted_deprecations.add(name)
             current = (current[0] or 'DEFAULT', current[1])
             # NOTE(bnemec): Not using versionutils for this to avoid a
@@ -1787,6 +1839,7 @@ class _Namespace(argparse.Namespace):
         :param names: a list of (section, name) tuples
         :param multi: a boolean indicating whether to return multiple values
         :param positional: whether this is a positional option
+        :param current_name: current name in tuple being checked
         """
         try:
             return self._get_cli_value(names, positional)
@@ -2062,8 +2115,8 @@ class ConfigOpts(collections.Mapping):
         as an attribute of this object.
 
         :param opt: an instance of an Opt sub-class
-        :param cli: whether this is a CLI option
         :param group: an optional OptGroup object or group name
+        :param cli: whether this is a CLI option
         :return: False if the opt was already registered, True otherwise
         :raises: DuplicateOptError
         """
@@ -2320,7 +2373,8 @@ class ConfigOpts(collections.Mapping):
         logger.log(lvl, "*" * 80)
         logger.log(lvl, "Configuration options gathered from:")
         logger.log(lvl, "command line args: %s", self._args)
-        logger.log(lvl, "config files: %s", self.config_file)
+        logger.log(lvl, "config files: %s",
+                   hasattr(self, 'config_file') and self.config_file or [])
         logger.log(lvl, "=" * 80)
 
         def _sanitize(opt, value):
@@ -2624,12 +2678,12 @@ class ConfigOpts(collections.Mapping):
             self._check_required_opts(namespace)
 
         except SystemExit as exc:
-            LOG.warn("Caught SystemExit while reloading configure files "
-                     "with exit code: %d", exc.code)
+            LOG.warning("Caught SystemExit while reloading configure files "
+                        "with exit code: %d", exc.code)
             return False
         except Error as err:
-            LOG.warn("Caught Error while reloading configure files: %s",
-                     err)
+            LOG.warning("Caught Error while reloading configure files: %s",
+                        err)
             return False
         else:
             self._namespace = namespace
@@ -2727,10 +2781,13 @@ class ConfigOpts(collections.Mapping):
             """Construct a StrSubWrapper object.
 
             :param conf: a ConfigOpts object
+            :param group: an OptGroup object
+            :param namespace: the namespace object that retrieves the option
+                              value from
             """
             self.conf = conf
-            self.namespace = namespace
             self.group = group
+            self.namespace = namespace
 
         def __getitem__(self, key):
             """Look up an opt value from the ConfigOpts object.
